#define INPUTC

#include "main.h"
#include "input.h"


/*
#define DEBUG_INPUTC_0
#define DEBUG_READ_VIEWING_DATA
*/

/*
 * <> -- only one item or sturcture.
 * {} -- repeat the item or structure as needed.
 * [] -- No input required.
 * Each item is on seperate line.
 *
 * Input Data file structure:
 *    <command> ::= <COMMAND_TOKEN> <"string">
 *    <environment> ::= <surface specification> <number of surfaces>
 *                      {<surface list>} <Viewing parameters>
 *    <Surface specification> ::= <number of vertices>
 *    <number of surfaces> ::= <6 - TOTAL_SURFACES_PER_ENV>
 *    <surface list> ::= {<number of edges> {<vertex list>}} <surface property>
 *    <vertex list> ::= {<x> <y> <z>}
 *    <surface property> ::= <emitted diffuse intensity> <reflectance>
 *    <emitted diffuse intensity> ::= <red intensity> <green intensity>
 *                                    <blue intensity>
 *    <reflectance> ::= <red reflectance><green reflectance><blue reflectance>
 *
 *    <Image parameters> ::= <raster window> [raster depth]
 *    <raster window> ::= <x_resolution y_resolution>
 *    <raster depth> ::= <boolsizeof(int:Image_Shape)>
 *
 *    <viewing parameters> ::= <view plane distance> <viewplane window>
 *                             <number of view point> {<view plane normal>
 *                             <up vector> <view point>}
 *    <view plane distance> ::= <distance>
 *    <viewplane window> ::= <lower_left_corner> <upper_right_corner>
 *    <upper_left_corner> ::= <x_min y_max>
 *    <lower_right_corner> ::= <x_max y_mix>
 *    <number of view point> ::= <1>
 *    <view plane normal> ::= <x> <y> <z>
 *    <up vector> ::=  <x> <y> <z>
 *    <view point> ::= <x> <y> <z>
 *
 *    author: s.n.pattanaik modified by Alvin Leung on 10-23-91
 *
 */

world *Read_World_Data(char *Data_FileName)
{
   char              *Subroutine_Name = "Read_World_Data()";
   FILE              *DataFile;
   world             *World;
   char               Buffer[READ_BUFFER_LEN];
   int                Max_Num_Vertices;
   int                Num_Polygons;
   polygon           *Polygons;
   polygon           *Polygon;
   vertex            *Vertices;
   int                mask;
   int                i;
   int                j;
   int                junk;

   if ((DataFile = fopen(Data_FileName, "r")) == NULL)
   {
      printf("%s -- Error. Connot open data file (%s).\n",
             Subroutine_Name, Data_FileName);
      exit(1);
   }

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%d", &Num_Polygons);

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%d", &Max_Num_Vertices);

#ifdef DEBUG_Read_World_Data
   printf("Num_Polygons = %d, Num_Vertices = %d\n", Num_Polygons, Num_Vertices);
#endif

   World = Get_World_Info(Num_Polygons, Max_Num_Vertices);

   Polygons = World -> Polygons;
   Vertices = World -> Vertices;

   mask = VERTEX_MASK(x) | VERTEX_MASK(y) | VERTEX_MASK(z)
          | VERTEX_MASK(r) | VERTEX_MASK(g) | VERTEX_MASK(b);

   for (i = 0; i < Num_Polygons; i++)
   {
      Polygon = &(Polygons[i]);
#if 0
      Polygon -> max_n = Max_Num_Vertices;
#endif
      Read_line(DataFile, Buffer, READ_BUFFER_LEN);
      sscanf(Buffer, "%d%*c", &(Polygon -> n));

      Polygon -> mask = mask;

      Vertices = Polygon -> Vertices;

      for (j = 0; j < Max_Num_Vertices; j++)
      {
         Read_line(DataFile, Buffer, READ_BUFFER_LEN);
         sscanf(Buffer, "\t%lf %lf %lf%*c",
                &(Vertices[j].x), &(Vertices[j].y), &(Vertices[j].z));
         Vertices[j].sx = Vertices[j].x;
         Vertices[j].sy = Vertices[j].y;
         Vertices[j].sz = Vertices[j].z;

#ifdef DEBUG_Read_World_Data
         printf("\t%lf %lf %lf\n", Vertices[j].x, Vertices[j].y, Vertices[j].z);
#endif
         Read_line(DataFile, Buffer, READ_BUFFER_LEN);
         sscanf(Buffer, "\t\t%lf %lf %lf", &(Vertices[j].r),
                &(Vertices[j].g), &(Vertices[j].b));

#ifdef DEBUG_Read_World_Data
         printf("\t\t%lf %lf %lf\n",
                Vertices[j].r, Vertices[j].g, Vertices[j].b);
#endif
      }
   }

   fclose(DataFile);
   return(World);
}

#if 0
void Read_Image_Data(FILE *DataFile, View_Info **Image_Data)
{
   int  Resolution_X;
   int  Resolution_Y;
   int  Depth;
   char Buffer[READ_BUFFER_LEN];

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%d, %d", &Resolution_X, &Resolution_Y);

   (*Image_Data) = Get_Image_Info(Resolution_X, Resolution_Y);
}
#endif

view *Read_View_Data(char *Data_FileName)
{
   char     *Subroutine_Name = "Read_View_Data()";
   char      Buffer[READ_BUFFER_LEN];
   FILE     *DataFile;
   view     *View;
   double    View_direction[3];
   double    Up[3];
   double    Projection;
   eye      *Eye;
   int       Num_Eye_Points;
   int       i;

   if ((DataFile = fopen(Data_FileName, "r")) == NULL)
   {
      printf("%s -- Error. Connot open data file (%s).\n",
             Subroutine_Name, Data_FileName);
      exit(1);
   }

   View = Get_View_Info();

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%lf", &(View -> Distance_to_Screen));

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%lf", &(View -> Near_Plane_Distance));

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%lf", &(View -> Convergence_Distance));

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%d %d", &(View -> Screen.LL_x), &(View -> Screen.LL_y));

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%d %d", &(View -> Screen.UR_x), &(View -> Screen.UR_y));

   Read_line(DataFile, Buffer, READ_BUFFER_LEN);
   sscanf(Buffer, "%d", &Num_Eye_Points);

   View -> Num_Eye_Points = Num_Eye_Points;

   View -> Eye = Eye = Get_Eye_Info(Num_Eye_Points);

   for (i = 0; i < Num_Eye_Points; i++)
   {
      Read_line(DataFile, Buffer, READ_BUFFER_LEN);
      sscanf(Buffer, "%lf %lf %lf", &View_direction[I_HAT],
             &View_direction[J_HAT], &View_direction[K_HAT]);

      Vector_Normalize(View_direction);

      Eye[i].View_direction[I_HAT] = View_direction[I_HAT];
      Eye[i].View_direction[J_HAT] = View_direction[J_HAT];
      Eye[i].View_direction[K_HAT] = View_direction[K_HAT];

#ifdef DEBUG_READ_VIEWING_DATA
      printf("View_direction = (%lf %lf %lf)\n",
             Eye[i].View_direction[I_HAT],
             Eye[i].View_direction[J_HAT],
             Eye[i].View_direction[K_HAT]);
#endif

      Read_line(DataFile, Buffer, READ_BUFFER_LEN);
      sscanf(Buffer, "%lf %lf %lf", &Up[I_HAT],
             &Up[J_HAT], &Up[K_HAT]);

#ifdef DEBUG_READ_VIEWING_DATA
      printf("Buffer = %s\n", Buffer);
#endif

      Vector_Normalize(Up);

#ifdef DEBUG_READ_VIEWING_DATA
      printf("Up = (%lf, %lf, %lf)\n", Up[I_HAT],
             Up[J_HAT], Up[K_HAT]);
#endif

      Projection = Vectors_Dot(Up, View_direction);

#ifdef DEBUG_READ_VIEWING_DATA
      printf("Up = (%lf, %lf, %lf)\n", Up[I_HAT],
             Up[J_HAT], Up[K_HAT]);

      printf("View_direction = (l%f, %lf, %lf)\n", View_direction[I_HAT],
             View_direction[J_HAT], View_direction[K_HAT]);

      printf("Projection = %g\n", Projection);
#endif

      if (Projection >= SMALL_NUMBER)
      {
         Up[I_HAT] -= Projection * View_direction[I_HAT];
         Up[J_HAT] -= Projection * View_direction[J_HAT];
         Up[K_HAT] -= Projection * View_direction[K_HAT];
         Vector_Normalize(Up);
      }

      Eye[i].Up[I_HAT] = Up[I_HAT];
      Eye[i].Up[J_HAT] = Up[J_HAT];
      Eye[i].Up[K_HAT] = Up[K_HAT];

#ifdef DEBUG_READ_VIEWING_DATA
      printf("Up = (%lf, %lf, %lf)\n",
             Eye[i].Up[I_HAT],
             Eye[i].Up[J_HAT],
             Eye[i].Up[K_HAT]);
#endif

      Read_line(DataFile, Buffer, READ_BUFFER_LEN);
      sscanf(Buffer, "%lf %lf %lf",
             &(Eye[i].Location[X]),
             &(Eye[i].Location[Y]),
             &(Eye[i].Location[Z]));

   }
   fclose(DataFile);
   return(View);
}

void Read_line(FILE *DataFile, char *Buffer, int Buffer_Len)
{
   char *Subroutine_Name = "Read_line()";

   do {
     if (fgets(Buffer, Buffer_Len, DataFile) == NULL) {
        printf("%s -- Error. EOF is encountered.\n", Subroutine_Name);
        exit(1);
     }
   } while ((Buffer[0] == COMMAND_TOKEN) || (Buffer[0] == EOLN));
}

